#ifdef WIN32
#include <windows.h>
#endif

#include "GL/glew.h"
#include "GL/glut.h"
#include "glutil.h"
#include "vecmath.h"
#include <fstream>

// The shaderProgram holds the vertexShader and fragmentShader
GLhandleARB vertexShader, fragmentShader, shaderProgram;

// The vertexArrayObject here will hold the pointers to 
// the vertex data (in vertBuffer) and color data per vertex (in colorBuffer)
GLuint vertBuffer, colorBuffer, vertexArrayObject;	

// In this simple application, we only use the combined modelViewProjectionMatrix
// (which should be equal to modelViewMatrix * projectionMatrix) 
Mtx4f modelViewMatrix, projectionMatrix, modelViewProjectionMatrix;

static void initGL()
{
	//******* Load Extensions ************
	glewInit();  


	//******** Create Triangle ************

	// Define the positions for each of the three points of the triangle
	float verts[] = {
	//	 X      Y     Z
		  0.0f,   0.5f, 1.0f,		// v0
		 -0.5f,  -0.5f, 1.0f,		// v1
		  0.5f,  -0.5f, 1.0f		// v2
	};
	
	// Define the colors for each of the three points of the triangle
	float colors[] = {
	//  R     G		B
		1.0f, 1.0f, 1.0f,		// White
		1.0f, 1.0f, 1.0f,		// White
		1.0f, 1.0f, 1.0f		// White

//		1.0f, 0.0f, 0.0f,		// Red
//		0.0f, 1.0f, 0.0f,		// Green
//		0.0f, 0.0f, 1.0f		// Blue
	};
	
	glGenBuffersARB( 1, &vertBuffer );											// Create a handle for the vertex position buffer
	glBindBufferARB( GL_ARRAY_BUFFER, vertBuffer );								// Set the newly created buffer as the current one
	glBufferDataARB( GL_ARRAY_BUFFER, sizeof(verts), verts, GL_STATIC_DRAW );	// Send the vetex position data to the current buffer
	
	glGenBuffersARB( 1, &colorBuffer );											// Create a handle for the vertex color buffer
	glBindBufferARB( GL_ARRAY_BUFFER, colorBuffer );							// Set the newly created buffer as the current one
	glBufferDataARB( GL_ARRAY_BUFFER, sizeof(colors), colors, GL_STATIC_DRAW );	// Send the vertex color data to the current buffer
	CHECK_GL_ERROR();
	//**************************************


	//**************Create Shaders******************

	char *vs = NULL,*fs = NULL,*fs2 = NULL;

	vertexShader = glCreateShaderObjectARB(GL_VERTEX_SHADER);
	fragmentShader = glCreateShaderObjectARB(GL_FRAGMENT_SHADER);

	vs = textFileRead("simple.vert"); // On mac, use "../../simple.vert"
	fs = textFileRead("simple.frag"); // On mac, use "../../simple.frag"

	const char * vv = vs;
	const char * ff = fs;

	glShaderSource(vertexShader, 1, &vv,NULL);
	glShaderSource(fragmentShader, 1, &ff,NULL);

	free(vs);free(fs);

	glCompileShader(vertexShader);
	int errorFlag = -1;
	glGetObjectParameterivARB(vertexShader, GL_OBJECT_COMPILE_STATUS_ARB, &errorFlag);
	if(!errorFlag) {
		std::string err = GetShaderInfoLog(vertexShader);
		MessageBox(NULL, err.c_str(), "OpenGL Tutorial - Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	glCompileShader(fragmentShader);
	glGetObjectParameterivARB(fragmentShader, GL_OBJECT_COMPILE_STATUS_ARB, &errorFlag);
	if(!errorFlag) {
		std::string err = GetShaderInfoLog(fragmentShader);
		MessageBox(NULL, err.c_str(), "OpenGL Tutorial - Error", MB_OK | MB_ICONEXCLAMATION);
		return;
	}

	shaderProgram = glCreateProgramObjectARB();
	glAttachObjectARB(shaderProgram, fragmentShader);
	glAttachObjectARB(shaderProgram, vertexShader);

	glBindAttribLocation(shaderProgram, 0, "vertex"); 
	glBindAttribLocation(shaderProgram, 1, "color");

	glLinkProgramARB(shaderProgram);

	// Now that the shader program has been linked, we no longer need these two intermediate objects and should delete them
	// Only the shader program needs to be retained for use while rendering
	glDeleteObjectARB( vertexShader );
	glDeleteObjectARB( fragmentShader );
	CHECK_GL_ERROR();
	//**********************************************


	glUseProgramObjectARB( shaderProgram );					// Set the shader program to use for this draw call

	//******* Connect triangle data with the vertex shader*******
	glGenVertexArrays(1, &vertexArrayObject);
	glBindVertexArray(vertexArrayObject);
	CHECK_GL_ERROR();

	glBindBufferARB( GL_ARRAY_BUFFER_ARB, vertBuffer );		// Connects vertBuffer to vertexArrayObject and also makes vertBuffer active for the next command below.
	glVertexAttribPointerARB(0, 3, GL_FLOAT, false/*normalized*/, 0/*stride*/, 0/*offset*/ );	

	glBindBufferARB( GL_ARRAY_BUFFER_ARB, colorBuffer );	// Connects colorBuffer to vertexArrayObject and also makes vertBuffer active for the next command below.
	glVertexAttribPointerARB(1, 3, GL_FLOAT, false/*normalized*/, 0/*stride*/, 0/*offset*/ );
	CHECK_GL_ERROR();

	glEnableVertexAttribArray(0);
	glEnableVertexAttribArray(1);
	CHECK_GL_ERROR();
	//***********************************************************

	// Set modelViewProjectionMatrix in vertex shader
	int loc = glGetUniformLocation(shaderProgram, "modelViewProjectionMatrix");
	modelViewProjectionMatrix.loadIdentity();
	glUniformMatrix4fv(loc, 1, false/* transpose */, &(modelViewProjectionMatrix.mtx[0][0]));
	CHECK_GL_ERROR();
}


static void drawScene(void)
{
	// Shader Program
	glUseProgramObjectARB( shaderProgram );			// Set the shader program to use for this draw call
	CHECK_GL_ERROR();
	glBindVertexArray(vertexArrayObject);
	CHECK_GL_ERROR();

	glDrawArrays( GL_TRIANGLES, 0, 3 );				// Render three points with the current sources
	CHECK_GL_ERROR();

	glUseProgramObjectARB( 0 );						// "unsets" the current shader program. Not really necessary.
	CHECK_GL_ERROR();

	}

void display(void)
{
	glClearColor(0.2,0.2,0.8,1.0);						// Set clear color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // Clears the color buffer and the z-buffer

	int w = glutGet((GLenum)GLUT_WINDOW_WIDTH);
	int h = glutGet((GLenum)GLUT_WINDOW_HEIGHT);
	glViewport(0, 0, w, h);						// Set viewport
		
	glDisable(GL_CULL_FACE);
	drawScene();

	glutSwapBuffers();  // swap front and back buffer. This frame will now been displayed.
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	/* open window of size 800x600 with double buffering, RGB colors, and Z-buffering */
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(512,512);
	glutCreateWindow("Test App");
	/* the display function is called once when the gluMainLoop is called,
    * but also each time the window has to be redrawn due to window 
    * changes (overlap, resize, etc).  
    */
	glutDisplayFunc(display);	// Set the main redraw function
	
	initGL();

	glutMainLoop();  /* start the program main loop */
	
	return 0;          
}
